<?php

GFForms::include_addon_framework();

class GWPMergeTags extends GFAddOn {

    protected $_version = GWP_MERGETAGS_VERSION;
    protected $_min_gravityforms_version = '1.9';
    protected $_slug = 'gravitywp-merge-tags';
    protected $_path = 'gravitywp-merge-tags/gravitywp-merge-tags.php';
    protected $_full_path = __FILE__;
    protected $_title = 'Merge Tags';
    protected $_short_title = 'Merge Tags';

    private static $_instance = null;
	
	public function __construct() {
		$this->_capabilities_plugin_page = 'gravityforms_edit_forms';
		$this->_capabilities_settings_page = 'gravityforms_edit_settings';
		parent::__construct();
    }

    /**
     * Get an instance of this class.
     *
     * @return GWPMergeTags
     */
    public static function get_instance() {
        if ( self::$_instance == null ) {
            self::$_instance = new GWPMergeTags();
        }

        return self::$_instance;
    }

    // # ADMIN FUNCTIONS -----------------------------------------------------------------------------------------------

    /**
     * Creates a custom admin page for GravityWP - Merge Tags
     */
    public function plugin_page() {
        if (!empty($_GET['id'])) {
            // Include gforms admin styles.
			wp_print_styles( array( 'jquery-ui-styles', 'gform_admin' ) );

            $active_tab = isset( $_GET[ 'tab' ] ) ? $_GET[ 'tab' ] : 'merge-tags';
            $form = RGFormsModel::get_form_meta($_GET['id']);

            function gwp_create_menu_item( $slug, $title ) {
                $url = 'href="?page=gravitywp-merge-tags&id=' . $_GET['id'];
                $active = '';
                if ($_GET['tab'] == $slug ) {
                    $active = ' nav-tab-active';
                }
                echo '<a ' . $url . '&tab=' . $slug . '" class="nav-tab' . $active;
                echo '">' . $title . '</a>';
            }

            // Add form title.
			GFCommon::form_page_title( GFAPI::get_form( $_GET['id'] ) );

			// Add top toolbar.
			GFForms::top_toolbar();

            // Create menu.
            echo '<h2>' . $form["title"] . '</h2><h2 class="nav-tab-wrapper">';

                // Create menu items.
                $var_merge_tags = __( 'Merge Tags', 'gravitywp-merge-tags' );
                gwp_create_menu_item( 'merge-tags', $var_merge_tags );

                $var_merge_tags_advanced = __( 'Advanced', 'gravitywp-merge-tags' );
                gwp_create_menu_item( 'merge-tags-advanced', $var_merge_tags_advanced );

				$var_merge_tags_admin = __( 'Admin', 'gravitywp-merge-tags' );
                gwp_create_menu_item( 'merge-tags-admin', $var_merge_tags_admin );

                $var_merge_tags_standard = __( 'Meta', 'gravitywp-merge-tags' );
                gwp_create_menu_item( 'standard-merge-tags', $var_merge_tags_standard );

                if (class_exists('Gravity_Flow_API')) {
                    $var_merge_tags_gravity_flow = __( 'Workflow', 'gravitywp-merge-tags' );
                    gwp_create_menu_item( 'gravity-flow', $var_merge_tags_gravity_flow );
                }
				
				$var_merge_tags_all_fields = __( 'All Fields', 'gravitywp-merge-tags' );
                gwp_create_menu_item( 'all-fields', $var_merge_tags_all_fields );

            echo '</h2>';

            // Create Tabs
            $fields = array();

            // Tab Merge Tags
            if( $active_tab == 'merge-tags' ) {
                echo    "<p></p><table class='wp-list-table widefat striped' cellspacing='0'><thead>
                        <tr><th>" . __( 'Merge Tags', 'gravitywp-merge-tags' ) . "</th></tr>
                        </thead><tbody>";

                if(is_array($form["fields"])){
                    foreach($form["fields"] as $field){
                        if(isset($field["inputs"]) && is_array($field["inputs"])){

                            foreach($field["inputs"] as $input)
                                echo "<tr><td>&#123;" . GFCommon::get_label($field, $input["id"]) . ":" . $input["id"] . "&#125;</td></tr>";
                        }
                        else if(!rgar($field, 'displayOnly')){
                            echo "<tr><td>&#123;" . GFCommon::get_label($field) . ":" . $field["id"] . "&#125;</td></tr>";
                        }
                    }
                }
                echo "</tbody></table>";
            }

            // Tab Merge Tags Advanced
            if( $active_tab == 'merge-tags-advanced' ) {
                echo    "<p></p>
                         <table class='wp-list-table widefat striped' cellspacing='0'><thead>
                            <tr>
                                <th>" . __( 'Field Label', 'gravitywp-merge-tags' ) . "</th>
                                <th>" . __( 'Merge Tag', 'gravitywp-merge-tags' ) . "</th>
                                <th>" . __( 'Merge Tags (short)', 'gravitywp-merge-tags' ) . "</th>
                                <th>" . __( 'Field Type', 'gravitywp-merge-tags' ) . "</th>
                            </tr></thead><tbody>";

                if(is_array($form["fields"])){
                    foreach($form["fields"] as $field){
                        if(isset($field["inputs"]) && is_array($field["inputs"])){
                            foreach($field["inputs"] as $input)
                                echo '<tr>
                                        <td>' . GFCommon::get_label($field, $input["id"]) . '</td>
                                        <td>&#123;' . GFCommon::get_label($field) . ':' . $field["id"] . '&#125;</td>
                                        <td>&#123;:' . $input["id"] . '&#125;</td>
                                        <td>' . RGFormsModel::get_input_type($field) . '</td>
							          </tr>';
                        }
                        else if(!rgar($field, 'displayOnly')){
                            echo '<tr>
                                    <td>' . GFCommon::get_label($field) . '</td>
                                    <td>&#123;' . GFCommon::get_label($field) . ':' . $field["id"] . '&#125;</td>
                                    <td>&#123;:' . $field["id"] . '&#125;</td>
                                    <td>' . RGFormsModel::get_input_type($field) . '</td>
                                  </tr>';
                        }
                    }
                }

                echo "</tbody></table>";

            }

            // Tab Merge Tags Admin
            if( $active_tab == 'merge-tags-admin' ) {
                echo "<p></p>
                        <table class='wp-list-table widefat striped' cellspacing='0'>
                        <thead><tr>
                                <th>" . __( 'Admin Label', 'gravitywp-merge-tags' ) . "</th>
                                <th>" . __( 'Merge Tag', 'gravitywp-merge-tags' ) . "</th>
                                <th>" . __( 'Field Label', 'gravitywp-merge-tags' ) . "</th>
                                <th>" . __( 'Populate', 'gravitywp-merge-tags' ) . "</th>
                                <th>" . __( 'CSS', 'gravitywp-merge-tags' ) . "</th>
                                <th>" . __( 'Field Type', 'gravitywp-merge-tags' ) . "</th>
                               </tr>
                        </thead><tbody>";
                $pre_fill_url       = '/?';
                $pre_fill_url_excel = '"/?';
                if(is_array($form["fields"])){
                    foreach($form["fields"] as $field){

                        if(isset($field["inputs"]) && is_array($field["inputs"])){
                            foreach($field["inputs"] as $input)
                                echo '<tr>
							            <td>' . $field["adminLabel"] . '</td>
							            <td>&#123;' . $field["adminLabel"] . ':' . $input["id"] . '&#125;</td>
							            <td>' . $input["label"] . '</td>
							            <td>' . $field["inputName"] . '</td>
							            <td>' . $field["cssClass"] . '</td>
							            <td>' . RGFormsModel::get_input_type($field) . '</td>
							          </tr>';

                        } else if (!rgar($field, 'displayOnly')){
                            echo '<tr>
						            <td>' . $field["adminLabel"] . '</td>
						            <td>&#123;' . $field["adminLabel"] . ':' . $field["id"] . '&#125;</td>
						            <td>' . $field["label"] . '</td>
						            <td>' . $field["inputName"] . '</td>
						            <td>' . $field["cssClass"] . '</td>
						            <td>' . RGFormsModel::get_input_type($field) . '</td>
						          </tr>';
                        }
                        if ( ! empty( $field['inputName'] ) ) {
							$pre_fill_url       .= $field['inputName'] . '=MERGETAG&';
							$pre_fill_url_excel .= $field['inputName'] . '="&CELL&"';
						}
                    }
                }
                echo "<thead><tr><th colspan='6'>" . __( 'Url query string template for dynamic population', 'gravitywp-merge-tags' ) . '</th></tr></thead>';
				echo "<tr><td colspan='6'>" . rtrim( $pre_fill_url, '&' ) . '</tr></td>';
				echo "<thead><tr><th colspan='6'>" . __( 'Excel formula for generating url query string', 'gravitywp-merge-tags' ) . '</th></tr></thead>';
				echo '<tr><td colspan="6">' . rtrim( $pre_fill_url_excel, '&"' ) . '</tr></td>';
                echo "</tbody></table>";

            }

            // Tab Gravity Flow
            if (class_exists('Gravity_Flow_API')) {

                if( $active_tab == 'gravity-flow' ) {
                    $gwp_workflow = new Gravity_Flow_API( $_GET['id'] );
                    $gwp_workflow_steps = $gwp_workflow->get_steps();
                    echo "<p></p>
                            <table class='wp-list-table widefat striped' cellspacing='0'>
                             <thead>
                              <tr>
                              <th>" . __( 'Gravity Flow Step', 'gravitywp-merge-tags' ) . '</th>
                              <th>' . __( 'ID', 'gravitywp-merge-tags' ) . '</th>
                              <th>' . __( 'Type', 'gravitywp-merge-tags' ) . '</th>
                              <th>' . __( 'Active', 'gravitywp-merge-tags' ) . '</th>
                              <th>' . __( 'Conditions', 'gravitywp-merge-tags' ) . '</th>
                              <th>' . __( 'Next Step (id)', 'gravitywp-merge-tags' ) . "</th>
                                <th>" . __( 'Count Shortcode', 'gravitywp-merge-tags' ) . "</th>
                              </tr>
                             </thead>
                            <tbody>";

                    foreach ( $gwp_workflow_steps as $step ) {

                        $feed_meta            = $step->get_feed_meta();
                        $is_condition_enabled = rgar( $feed_meta, 'feed_condition_conditional_logic' ) == true;
                        $logic                = rgars( $feed_meta, 'feed_condition_conditional_logic_object/conditionalLogic' );
                        $condition_str        = '';

                        if ( $is_condition_enabled && ! empty( $logic ) ) {
                            if ( is_array( $logic['rules'] ) ) {
                                if ( count( $logic['rules'] ) > 1 ) {
                                    $condition_str = $logic['logicType'] . ':<br>';
                                }
                                foreach ( $logic['rules'] as $rule ) {
                                    $condition_str .= GFAPI::get_field( $_GET['id'], $rule['fieldId'] )->label . ':' . implode( ' -- ', $rule ) . '<br>';
                                }
                            }
                        }
                        $next_step_str = rgar( $feed_meta, 'destination_complete' );
                        echo '<tr>
                                <td>' . '<a href="/wp-admin/admin.php?page=gf_edit_forms&view=settings&subview=gravityflow&id=' . $_GET['id'] . '&fid=' . $step->get_id() . '">' . $step->get_name() . '</a></td>
                                <td>' . $step->get_id() . '</td>
                                <td>' . $step->get_type() . '</td>
                                <td>' . ( ( $step->is_active() == '1' ) ? '&#10003;' : '&#10007;' ) . '</td>
                                <td>' . $condition_str . '</td> 
                                <td>' . $next_step_str . '</td>
                                <td>&#91; gravitywp_count formid=&quot;' . $_GET['id'] . '&quot; workflow_step=&quot;' . $step->get_id() . '&quot;&#93;</td>
                                </tr>';
                    }
                    echo "</tbody></table>";
                }
            }

            // Tab Gravity Flow
            if( $active_tab == 'standard-merge-tags' ) {
                echo "<style>.gwp_div {width:32%; min-width: 300px; margin-right:1%; float:left !important;} .widefat {clear: none; }</style>";
                echo "<p></p>
                        <table class='wp-list-table widefat striped gwp_div' cellspacing='0'>
                         <thead>
                          <tr>
                           <td>Gravity Forms Merge Tags</td></tr>
                          </thead>
                         <tbody>";

                $gwp_standard_mergetags = array(
                    'all_fields',
                    'entry_id',
                    'entry_url',
                    'form_title',
                    'date_mdy',
                    'date_dmy',
                    'embed_url',
                    'ip',
                    'user_agent',
                    'referer',
                    'admin_email',
                    'currency',
                    'date_created',
                    'approval_status',
                    'is_fulfilled',
                    'payment_amount',
                    'payment_date',
                    'payment_method',
                    'payment_status',
                    'transaction_id',
                    'transaction_type',
                    'all_fields:admin',
                    'all_fields:value',
                    'all_fields:empty',
                    'all_fields:noadmin',
                    'all_fields:nohidden'
                );

                foreach ($gwp_standard_mergetags as $gwp_standard_mergetag) {
                    echo "<tr>
                            <td>&#123;" . $gwp_standard_mergetag . "&#125;</td>
                          </tr>";
                }

                echo "</tbody></table>";

                // Merge tags for GravityView
                echo "<table class='wp-list-table widefat striped gwp_div' cellspacing='0'>
                        <thead>
                         <tr>
                          <td><a href='https://gravityview.co/?ref=115' target='_blank'>GravityView</a></td>
                         </tr>
                        </thead>
                       <tbody>";

                $gwp_gv_mergetags = array(
                    'get',
                    'approval_status',
                    'approval_status:label',
                    'approval_status:value',
                    'date_created',
                    'date_created:time',
                    'date_created:format:m/d/Y',
                    'date_created:format:m/d/Y\ \a\t\ H\:i\:s',
                    'date_created:human',
                    'date_created:human:time',
                    'date_created:diff',
                    'date_created:diff:format:I submitted %s in the past',
                    'created_by',
                    'created_by:ID',
                    'user:display_name',
                    'user:user_email',
                    'user:user_login',
                    'user:[meta_key]'
                );

                foreach ($gwp_gv_mergetags as $gwp_gv_mergetag) {
                    echo "<tr>
                            <td>&#123;" . $gwp_gv_mergetag . "&#125;</td>
                          </tr>";
                }

                echo "</tbody></table>";

                // Merge tags for Gravityflow
                echo "<table class='wp-list-table widefat striped gwp_div' cellspacing='0'>
                        <thead>
                         <tr>
                          <td><a href='https://gravityflow.io/?ref=2' target='_blank'>Gravity Flow</td>
                         </tr>
                        </thead>
                       <tbody>";

                $gwp_gflow_mergetags = array(
                    'workflow_entry_link',
                    'workflow_entry_url',
                    'workflow_inbox_link',
                    'workflow_inbox_url',
                    'workflow_cancel_link',
                    'workflow_cancel_url',
                    'workflow_note',
                    'workflow_timeline',
                    'assignees',
                    'workflow_approve_link',
                    'workflow_approve_url',
                    'workflow_reject_token'
                );

                foreach ($gwp_gflow_mergetags as $gwp_gflow_mergetag) {
                    echo "<tr>
                            <td>&#123;" . $gwp_gflow_mergetag . "&#125;</td>
                          </tr>";
                }
                echo "</tbody></table>";
            }

            // All Fields Table tab
            if( $active_tab == 'all-fields' ) {
                echo "<style>table.fixed { table-layout: auto;  }</style>";
                echo "<p></p>
                        <table class='wp-list-table widefat striped fixed' cellspacing='0'>
                         <thead>
                          <tr>
                           <th class='gwp_title'>" . __( 'All Fields', 'gravitywp-merge-tags' ) .  " " . __( 'Type', 'gravitywp-merge-tags' ) . "</th>
                           <th>" . __( 'All Fields', 'gravitywp-merge-tags' ) . " " . __( 'Table', 'gravitywp-merge-tags' ) . "</th>
                          </tr>
                         </thead>
                        <tbody>
                         <tr>
                          <td>" . __( 'Replace All Fields Merge Tag', 'gravitywp-merge-tags' ) . "</td>
                          <td>&#60;table&#62;<br>";

                if(is_array($form["fields"])){
                    foreach($form["fields"] as $field){
                        if(isset($field["inputs"]) && is_array($field["inputs"])){

                            foreach($field["inputs"] as $input)
                                echo "&#60;tr&#62;&#60;td class='gwp-allfields-label' &#62;"
                                    . GFCommon::get_label($field, $input["id"])
                                    . "&#60;/td&#62;<br>&#60;td class='gwp-allfields-value' &#62;&#123;"
                                    . GFCommon::get_label($field, $input["id"])
                                    . ":"
                                    . $input["id"]
                                    . "&#125;&#60;/td&#62;&#60;/tr&#62;<br>";
                        }

                        else if(!rgar($field, 'displayOnly')){
                            echo "&#60;tr&#62;&#60;td class='gwp-allfields-label' &#62;"
                                . GFCommon::get_label($field)
                                . "&#60;/td&#62;<br>&#60;td class='gwp-allfields-value' &#62;&#123;"
                                . GFCommon::get_label($field)
                                . ":"
                                . $field["id"]
                                . "&#125;&#60;/td&#62;&#60;/tr&#62;<br>";
                        }
                    }
                }

                echo "&#60;/tbody&#62;&#60;/table&#62;</td></tr><tr><td>"
                    . __( 'Replace All Fields Merge Tag', 'gravitywp-merge-tags' )
                    .  " " . __( 'without fileuploads', 'gravitywp-merge-tags' )
                    . "</td><td>&#60;table&#62;<br>";

                if(is_array($form["fields"])){
                    foreach($form["fields"] as $field){
                        if(isset($field["inputs"]) && is_array($field["inputs"])){

                            foreach($field["inputs"] as $input)
                                if (RGFormsModel::get_input_type($field) != "fileupload") {
                                    echo "&#60;tr&#62;&#60;td class='gwp-allfields-label' &#62;"
                                        . GFCommon::get_label($field, $input["id"])
                                        . "&#60;/td&#62;<br>&#60;td class='gwp-allfields-value' &#62;&#123;"
                                        . GFCommon::get_label($field, $input["id"]) . ":" . $input["id"]
                                        . "&#125;&#60;/td&#62;&#60;/tr&#62;<br>";
                                }
                        }

                        else if(!rgar($field, 'displayOnly')){
                            if (RGFormsModel::get_input_type($field) != "fileupload") {
                                echo "&#60;tr&#62;&#60;td class='gwp-allfields-label' &#62;"
                                    . GFCommon::get_label($field)
                                    . "&#60;/td&#62;<br>&#60;td class='gwp-allfields-value' &#62;&#123;"
                                    . GFCommon::get_label($field) . ":"
                                    . $field["id"] . "&#125;&#60;/td&#62;&#60;/tr&#62;<br>";
                            }
                        }
                    }
                }

                echo "&#60;/tbody&#62;&#60;/table&#62;</td></tr></tbody></table>";
            }

        } else {
            $forms = RGFormsModel::get_forms( null, 'title' );
            echo "<table class='wp-list-table widefat striped' style='margin-top:10px;' cellspacing='0'><tr><thead><th>" .
                __( 'Select Form', 'gravitywp-merge-tags' )
                . "</th></tr></thead><tbody>";

            foreach( $forms as $form ):
                echo '<tr><td>
                        <a href="' . $_SERVER['REQUEST_URI'] . '&id=' . $form->id . '&tab=merge-tags' . '" target="_self">'
                          . $form->title .
                       '</a></td></tr>';
            endforeach;
            echo "</tbody></table>";

        }
    }
}